/*!
 * © 2014, 2022 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */

/* globals Character: true, Rules, jsPDF */
Character.makeOneCharacterPDF = function(theCharacter) {
    let pdf = new jsPDF("landscape", "pt", "a4");
    pdf.setProperties({
        title : theCharacter.name,
        keywords : "generated, linrodeth, character"
    });
    Character.makePDF(pdf,theCharacter,[1,2,3,4],true);
    return pdf;
};

Character.makeMultipleSkillPages = function(chars) {
    let pdf = new jsPDF("landscape", "pt", "a4");
    pdf.setProperties({
        title : "Multiple Character skill sheets",
        keywords : "generated, linrodeth, character"
    });

    let first = true;
    for(let ch of chars) {
        Character.makePDF(pdf,ch,[3,4],first);
        first = false;
    }
    return pdf;
};

Character.makePDF = function(pdf,theCharacter,pages,firstPage) {
   let layout = {};

    layout.namebottom = 30;
    layout.nameleft = 0;
    layout.nameright = 842;
    layout.nametop = 12;
    layout.left = 16;
    layout.midright = 400;
    layout.midleft = 442;
    layout.right = 842;
    layout.cellwidth = 32;
    layout.cellheight = 10;

    theCharacter.PDFfootertext = "Player: " + theCharacter.playerName + ", Rules Version: " + theCharacter.rulesVersion + ", Plugin Version: 3.226.0, Character version: " + theCharacter.characterVersion + ", Game Date:" + theCharacter.gameday + ", (last edit time " + theCharacter.lastEditDate + ", PDF created " + (new Date()).toLocaleString() + ")";

    if (pages.includes(1)) {
        if (!firstPage) pdf.addPage();
        Character.makeCoverPage(pdf,layout,theCharacter);
        firstPage = false;
    }
    if (pages.includes(2)) {
        if (!firstPage) pdf.addPage();
        Character.makeBiographyPage(pdf,layout,theCharacter);
        firstPage = false;
    }
    if (pages.includes(3)) {
        if (!firstPage) pdf.addPage();
        Character.makeAttributeTablesPage(pdf,layout,theCharacter);
        firstPage = false;
    }
    if (pages.includes(4)) {
        if (!firstPage) pdf.addPage();
        Character.makeSkillTablesPage(pdf,layout,theCharacter);
        firstPage = false;
    }
};

Character.PDFdrawSkillTable = function(pdf, title, skills, top, left, cellheight, cellwidth) {
    if (skills.length > 45) {
        cellheight--; // Cope with Goodgulf :)
    }
    if (skills.length > 50) {
        cellheight -= 0.5; // And again
    }

    let skillnamewidth = 81;
    let skilllevelwidth = 60;
    let levelsplit = 112;
    let vgutter = ((skills.length > 50) ? 2 : 3);
    let hgutter = 2;

    let mainleft = left + skillnamewidth + skilllevelwidth;
    let right = mainleft + Rules.adjectives.length * cellwidth;
    let bottom = top + (skills.length + 2) * cellheight;

    pdf.setLineWidth(1);
    pdf.line(mainleft, top, right, top);
    pdf.line(mainleft, top + cellheight, right, top + cellheight);
    pdf.setLineWidth(0.5);
    for (var x = 0; x < Rules.adjectives.length; x++) {
        pdf.line(mainleft + x * cellwidth, top, mainleft + x * cellwidth,
                bottom);
        pdf.setFontSize(Rules.adjectives[x].length < 12 ? 6 : 4.5); // Special case for "Embarrasing"
        pdf.textInBox(Rules.adjectives[x], {
            align : "center"
        }, mainleft + x * cellwidth + hgutter, top + cellheight - vgutter,
                mainleft + (x + 1) * cellwidth, top);
    }

    pdf.setLineWidth(1);
    pdf.line(left, top + cellheight, right, top + cellheight);
    pdf.line(left, top + 2 * cellheight, right, top + 2 * cellheight);
    pdf.line(left, bottom, right, bottom);
    pdf.line(left, top + cellheight, left, bottom);
    pdf.line(left + skillnamewidth, top + cellheight,
            left + skillnamewidth, bottom);
    pdf.line(mainleft, top, mainleft, bottom);
    pdf.line(right, top, right, bottom);
    pdf.setFontStyle("bold");
    pdf.setFontSize(7.5);
    pdf.text(left + hgutter, top + cellheight * 2 - 3, title);
    pdf.text(left + skillnamewidth + hgutter, top + cellheight * 2 - 3, "Level");
    for (x = 0; x < Rules.adjectives.length; x++) {
        pdf.textInBox(x.toString(), {
            align : "center"
        }, mainleft + x * cellwidth + hgutter, top + 2 * cellheight - vgutter,
                mainleft + (x + 1) * cellwidth, top + cellheight);
    }

    pdf.setLineWidth(0.5);
    pdf.setFontStyle("normal");
    let y = top + 3 * cellheight;
    for (let s = 0; s < skills.length; s++, y += cellheight) {
        if (s !== skills.length - 1) {
            pdf.line(left, y, right, y);
        }
        pdf.text(left + hgutter, y - 3, skills[s].skill);
        pdf.textInBox(skills[s].level.toString(), {
            align : "right"
        }, left + skillnamewidth+hgutter, y - vgutter, levelsplit, y - cellheight + 1);
        pdf.textInBox(Rules.adj(skills[s].level), {
            align : "left"
        }, levelsplit + hgutter, y - vgutter, mainleft, y - cellheight + 1);
        let perf = Rules.performance(skills[s].level);
        for (x = 0; x < Rules.adjectives.length; x++) {
            pdf.textInBox(perf[x], {
                align : "center"
            }, mainleft + x * cellwidth, y - vgutter, mainleft + (x + 1) * cellwidth, y - cellheight + 1);
        }
    }
};

Character.PDFdrawTable = function(pdf, content, widths, aligns, left, top, cellheight, borderweights) {
    let totalwidth = widths.reduce(function(x, y) {
        return x + y;
    }, 0);
    let hgutter = 2;
    pdf.setLineWidth(1);
    pdf.rect(left, top, totalwidth, content.length * cellheight);
    let offset = left + widths[0];
    let cell;
    for (cell = 1; cell < widths.length; cell++) {
        if (borderweights && borderweights[cell]) {
            pdf.setLineWidth(borderweights[cell]);
        }
        pdf.line(offset, top, offset, top + content.length * cellheight);
        offset += widths[cell];
    }
    pdf.setFontStyle("bold"); // Make top line bold
    pdf.setLineWidth(1);
    let y = top + cellheight;
    for (let line = 0; line < content.length; line++, y += cellheight) {
        if (line !== content.length - 1) {
            pdf.line(left, y, left + totalwidth, y);
        }
        let offset = left;
        for (cell = 0; cell < aligns.length; cell++) {
            pdf.textInBox(content[line][cell], {
                align : aligns[cell]
            }, offset + hgutter, y - 3, offset + widths[cell], y - cellheight);
            offset += widths[cell];
        }
        if (line === 0) {
            pdf.setFontStyle("normal");
            pdf.setLineWidth(0.5);
        }
    }
};

Character.PDFfooter = function(pdf,layout,theCharacter) {
    pdf.setFontSize(8);
    pdf.text(layout.left, 576, theCharacter.PDFfootertext);
};

Character.makeCoverPage= function(pdf,layout,theCharacter) {
    let skillsummmarycellheight = 12;
    if (theCharacter.sheet.skills.length > 32) { skillsummmarycellheight = 11;  }
    if (theCharacter.sheet.skills.length > 40) { skillsummmarycellheight = 10;  }
    if (theCharacter.sheet.skills.length > 45) { skillsummmarycellheight = 9;   }
    if (theCharacter.sheet.skills.length > 50) { skillsummmarycellheight = 8.5; }

    pdf.setFontSize(18);
    pdf.textInBox(theCharacter.name, {
        align : "center"
    }, layout.left, layout.namebottom, layout.midright, layout.nametop);
    pdf.setFontSize(skillsummmarycellheight-1);
    let content = [];
    let widths = [198,100,90];
    let aligns = ["left","left", "center"];
    let title = ["Skill", "Level", "Current Interest"];
    content.push(title);
    for (let s = 0; s < theCharacter.sheet.skills.length; s++) {
        let currentip = "";
        for(let cs = 0; cs < theCharacter.periods[theCharacter.periods.length-1].interests.length; cs++) {
            if (theCharacter.periods[theCharacter.periods.length-1].interests[cs].skill === theCharacter.sheet.skills[s].skill) {
                currentip = String(theCharacter.periods[theCharacter.periods.length-1].interests[cs].ip);
            }
        }
        let level = " " + theCharacter.sheet.skills[s].level + " (" + Rules.adj(theCharacter.sheet.skills[s].level)+")";
        content.push([theCharacter.sheet.skills[s].skill, level, currentip]);
    }
    Character.PDFdrawTable(pdf,content,widths,aligns,layout.left,layout.namebottom+8,skillsummmarycellheight);

    // front cover name, dates, stats, body, summary
    pdf.setFontSize(18);
    pdf.textInBox(theCharacter.name, {
        align : "center"
    }, layout.midleft, layout.namebottom, layout.right, layout.nametop);

    pdf.setFontSize(12);
    content = [];
    widths = [120,100];
    aligns = ["left","left"];
    title = ["Attribute","Level"];
    content.push(title);
    content.push(["Strength"," "+theCharacter.str+" ("+Rules.adj(theCharacter.str)+")"]);
    content.push(["Dexterity"," "+theCharacter.dex+" ("+Rules.adj(theCharacter.dex)+")"]);
    content.push(["Agility"," "+theCharacter.agi+" ("+Rules.adj(theCharacter.agi)+")"]);
    content.push(["Constitution"," "+theCharacter.con+" ("+Rules.adj(theCharacter.con)+")"]);
    content.push(["Appearance"," "+theCharacter.app+" ("+Rules.adj(theCharacter.app)+")"]);
    content.push(["Intellect"," "+theCharacter.int+" ("+Rules.adj(theCharacter.int)+")"]);
    content.push(["Wit"," "+theCharacter.wit+" ("+Rules.adj(theCharacter.wit)+")"]);
    content.push(["Will"," "+theCharacter.will+" ("+Rules.adj(theCharacter.will)+")"]);
    content.push(["",""]);
    content.push(["Interest Points"," "+theCharacter.interestPoints]);
    content.push(["Morale"," "+theCharacter.morale]);
    Character.PDFdrawTable(pdf,content,widths,aligns,layout.midleft,layout.namebottom+8,14);

    let midmidleft = (layout.midleft + layout.right)/2+50;
    let bodycentre = midmidleft + (layout.right-midmidleft-20)/2;
    let bodytop = layout.namebottom+8;
    pdf.setLineWidth(1);
    pdf.rect(midmidleft,layout.bodytop,layout.right-midmidleft-20,16*14);
    pdf.setLineWidth(0.5);
    pdf.lines([[32,0],[0,36],[42,0],[0,90],[-20,0],[0,-72],[-7,0],[0,144],[-24,0],[0,-72],[-14,0],[0,72],[-24,0],[0,-144],[-7,0],[0,72],[-20,0],[0,-90],[42,0],[0,-36]],bodycentre-16,bodytop+9);
    let locpic = function(location) {
        let containsShieldLocation = function(location) {
            return function(x) {
                return x === location;
            };
        };

        let points = theCharacter.armour.value;
        if (theCharacter.shieldLocations.some(containsShieldLocation(location))) {
            points += theCharacter.shield.value;
        }
        return theCharacter.woundPoints[location]+"/"+points.toString();
    };
    pdf.textInBox(locpic("head"),{align : "center", valign: "center"},bodycentre-16,bodytop+45,bodycentre+16,bodytop+9);
    pdf.textInBox(locpic("shieldArm"),{align : "center", valign: "center"},bodycentre+39,bodytop+135,bodycentre+57,bodytop+45);
    pdf.textInBox(locpic("swordArm"),{align : "center", valign: "center"},bodycentre-59,bodytop+135,bodycentre-39,bodytop+45);
    pdf.textInBox(locpic("chest"),{align : "center", valign: "center"},bodycentre-32,bodytop+111,bodycentre+32,bodytop+63);
    pdf.textInBox(locpic("abdomen"),{align : "center", valign: "center"},bodycentre-32,bodytop+135,bodycentre+32,bodytop+111);
    pdf.textInBox(locpic("rightLeg"),{align : "center", valign: "center"},bodycentre-32,bodytop+207,bodycentre-8,bodytop+135);
    pdf.textInBox(locpic("leftLeg"),{align : "center", valign: "center"},bodycentre+6,bodytop+207,bodycentre+32,bodytop+135);

    pdf.text("Gender: "+theCharacter.gender,layout.midleft,layout.namebottom+200);
    pdf.text("Birthday: "+theCharacter.birthday,layout.midleft,layout.namebottom+214);
    pdf.text("Game Date: "+theCharacter.gameday,layout.midleft,layout.namebottom+228);
    pdf.setFontSize(8);
    pdf.textInBox("wound points/armour",{align:"center"}, midmidleft,bodytop+16*14-1.5,layout.right-20,bodytop+16*14-9 );

    pdf.setFontSize(12);
    pdf.setFontStyle("bold");
    pdf.text("Description",layout.midleft,layout.namebottom+260);
    pdf.setFontStyle("normal");
    pdf.text(pdf.splitTextToSize(theCharacter.description,layout.right-layout.midleft),layout.midleft,layout.namebottom+274);

    Character.PDFfooter(pdf,layout,theCharacter);
};

Character.makeBiographyPage = function(pdf,layout,theCharacter) {
    let proposedptsize = 12;
    let retry, render = false, done = false;
    let pageoffset, lineoffset;

    while (!done) {
        pageoffset = layout.left;
        lineoffset = 28;
        if (render) { pdf.setFontSize(proposedptsize); }
        retry = false;
        for(let p=0; (p<theCharacter.periods.length); p++ ) {
            let desclines = pdf.splitTextToSize(theCharacter.periods[p].description,layout.midright-layout.left);
            if (lineoffset + proposedptsize*1.15*(desclines.length+2) > 560) {
                // switch page
                pageoffset += (layout.midleft-layout.left);
                lineoffset = 28;
                if (pageoffset > layout.right) {
                    proposedptsize--;
                    retry = true;
                    break;
                }
            }
            if (render) {
                pdf.setFontStyle("bold");
                pdf.text(theCharacter.periods[p].title+" ("+theCharacter.periods[p].startmonthtext+"-"+theCharacter.periods[p].endmonthtext+")",pageoffset,lineoffset);
                pdf.setFontStyle("normal");
                pdf.text(desclines,pageoffset,lineoffset+proposedptsize*1.15);
            }
            lineoffset += proposedptsize*1.15*(desclines.length+2);
        }
        if (render) { done = true; }
        if (!retry) { render = true; }
    }

    Character.PDFfooter(pdf,layout,theCharacter);
};

Character.makeAttributeTablesPage = function(pdf,layout,theCharacter) {
    pdf.setFontSize(18);
    pdf.textInBox(theCharacter.name, {
        align : "center"
    }, layout.nameleft, layout.namebottom, layout.nameright, layout.nametop);

    // Need to Fake up a skills array for the attributes
    let attributesAsSkills = [];
    for (let stat = 0; stat < Character.allstats.length; stat++) {
        attributesAsSkills.push({
            "skill" : Character.longstats[stat],
            "level" : theCharacter[Character.allstats[stat]]
        });
    }
    Character.PDFdrawSkillTable(pdf, "Attribute", attributesAsSkills, layout.namebottom+16, layout.left, layout.cellheight, layout.cellwidth);

    let i,w,x,weapon;
    let content = [];
    let widths = [ 81, 30, 30 ];
    let aligns = [ "left", "center", "center" ];
    let title = [ "Weapon", "Area", "Hits" ];
    for (x = 0; x < Rules.adjectives.length; x++) {
        title.push(x.toString());
        widths.push(layout.cellwidth);
        aligns.push("center");
    }
    content.push(title);
    let hasarea = false;
    let hashits = false;
    for (w = 0; w < theCharacter.favouredWeapons.length; w++) {
        weapon = theCharacter.favouredWeapons[w];
        if (weapon.ranged) {
            let area = weapon.area || "";
            let hits = weapon.hits || "";
            hasarea = hasarea || weapon.area || false;
            hashits = hashits || weapon.hits || false;
            let ranges = [ weapon.name, area, hits ];
            ranges = ranges.concat(weapon.ranges);
            for (i = 0; i < ranges.length; i++) {
                ranges[i] = (ranges[i] ? ranges[i].toString() : "");
            }
            content.push(ranges);
        }
    }
    pdf.text(layout.left + 3, 153, "Difficulty to hit at range");
    Character.PDFdrawTable(pdf, content, widths, aligns, layout.left, 156, layout.cellheight, [undefined, 1, 0.5, 1, 0.5 ]);

    content = [];
    widths = [ 81, 60 ];
    aligns = [ "left", "center" ];
    title = [ "Weapon", "Damage" ];
    for (x = 0; x < Rules.adjectives.length; x++) {
        title.push(x.toString());
        widths.push(layout.cellwidth);
        aligns.push("center");
    }
    content.push(title);
    let line = [ "Unarmed", "1" ];
    line = line.concat(Rules.damageTable[1]);
    content.push(line);
    for (w = 0; w < theCharacter.favouredWeapons.length; w++) {
        weapon = theCharacter.favouredWeapons[w];
        line = [ weapon.name, weapon.damage.toString() ];
        line = line.concat(Rules.damageTable[weapon.damage]);
        content.push(line);
    }
    Character.PDFdrawTable(pdf, content, widths, aligns, layout.left, 212, layout.cellheight, [undefined, 1, 1, 0.5 ]);

    content = [];
    content.push([ "Situation", "Mod", "Example for cooking" ]);
    content.push([ "Excellent", "+5", "Staffed restaurant kitchen" ]);
    content.push([ "Very good", "+3", "Skilled help" ]);
    content.push([ "Good", "+1", "Normal kitchen, some help" ]);
    content.push([ "Normal", "", "Normal kitchen" ]);
    content.push([ "Poor", "-1", "No spices or herbs available" ]);
    content.push([ "Very Poor", "-3", "Open fire, few ingredients" ]);
    content.push([ "Terrible", "-5", "No fire" ]);
    Character.PDFdrawTable(pdf, content, [ 41, 120 - 86, 225 - 120 ], [ "left", "center", "left" ], 45, 324, layout.cellheight);

    let modwidths = [ 90, 36 ];
    content = [];
    content.push([ "Melee situation", "Modifier" ]);
    content.push([ "Defending only", "+3" ]);
    content.push([ "At Opponents side", "+1" ]);
    content.push([ "Behind Opponent", "+2" ]);
    content.push([ "Unarmed versus blunt", "-1" ]);
    content.push([ "Unarmed versus blade", "-4" ]);
    content.push([ "Shorter weapon", "-1" ]);
    content.push([ "Each extra opponent", "-1"]);
    Character.PDFdrawTable(pdf, content, modwidths, [ "left", "center" ], 252, 324, layout.cellheight);

    content = [];
    content.push([ "Location Hit", "Melee", "Ranged" ]);
    content.push([ "Head", "91-00", "96-00" ]);
    content.push([ "Left Arm", "81-90", "86-95" ]);
    content.push([ "Right Arm", "71-80", "76-85" ]);
    content.push([ "Chest", "45-70", "51-75" ]);
    content.push([ "Abdomen", "33-44", "33-50" ]);
    content.push([ "Left Leg", "17-32", "17-32" ]);
    content.push([ "Right Leg", "01-16", "01-16" ]);
    Character.PDFdrawTable(pdf, content, [ 64, 32, 32 ], [ "left", "center", "center" ], 405, 324, layout.cellheight);

    content = [];
    content.push([ "Ranged situation", "Modifier" ]);
    content.push([ "Aimed Shot (2 rounds)", "+2" ]);
    content.push([ "Walking target", "-2" ]);
    content.push([ "Jogging target", "-3" ]);
    content.push([ "Running target", "-4" ]);
    content.push([ "Dodging target", "-5" ]);
    content.push([ "Light cover", "-1" ]);
    content.push([ "Medium cover", "-3" ]);
    content.push([ "Heavy cover", "-5" ]);
    Character.PDFdrawTable(pdf, content, modwidths, [ "left", "center" ], 252, 410, layout.cellheight);

    content = [];
    content.push([ "Modifiers for previous round", "Modifier" ]);
    content.push([ "Minor defeat last round", "-1" ]);
    content.push([ "Clear defeat last round", "-2" ]);
    content.push([ "Major defeat last round", "-3" ]);
    content.push([ "Devastating defeat last round", "-4" ]);
    Character.PDFdrawTable(pdf, content, [ 148, 32 ], [ "left", "center" ], 45, 414, layout.cellheight);

    content = [];
    content.push([ "Gap", "Type of win", "Wound Location" ]);
    content.push([ "1-3", "Minor", "None" ]);
    content.push([ "4-6", "Clear", "Random" ]);
    content.push([ "7-9", "Major", "Move 1 place, +2 damage" ]);
    content.push([ "10+", "Devastating", "Move 2 places, +5 damage" ]);
    Character.PDFdrawTable(pdf, content, [ 32, 120 - layout.left - 32, 225 - 120 ], [ "left", "left", "left" ], layout.left, 477 + layout.cellheight, layout.cellheight);
    pdf.setLineWidth(1);
    pdf.setFontStyle("bold");
    pdf.line(layout.left, 477 + layout.cellheight, layout.left, 477);
    pdf.line(layout.left, 477, 225, 477);
    pdf.line(225, 477, 225, 477 + layout.cellheight);
    pdf.text(layout.left+2, 477 + layout.cellheight - 3, "Performance Difference"); // +2 for the hgutter that drawTable used

    pdf.setLineWidth(1);
    pdf.rect(569, 324, 8 * layout.cellwidth, 8 * layout.cellheight);
    pdf.line(569, 324 + layout.cellheight, 569 + 2 * layout.cellwidth, 324 + layout.cellheight);
    pdf.line(569 + 2 * layout.cellwidth, 324, 569 + 2 * layout.cellwidth, 324 + layout.cellheight);
    pdf.setLineWidth(0.5);
    for (i = 1; i < 8; i++) {
        pdf.line(569 + i * layout.cellwidth, 324, 569 + i * layout.cellwidth, 324 + 8 * layout.cellheight);
        pdf.line(569, 324 + i * layout.cellheight, 569 + 8 * layout.cellwidth, 324 + i * layout.cellheight);
    }
    pdf.text(570, 324 + layout.cellheight - 3, "Morale");
    pdf.text(570 + layout.cellwidth, 324 + layout.cellheight - 3, theCharacter.morale.toString());

    content = [];
    content.push([ "Area", "Armour", "WP", "Armour" ]);
    var locline = function(name, location) {
        var containsShieldLocation = function(location) {
            return function(x) {
                return x === location;
            };
        };

        var armour = theCharacter.armour.name;
        var points = theCharacter.armour.value;
        if (theCharacter.shieldLocations.some(containsShieldLocation(location))) {
            armour += " + " + theCharacter.shield.name;
            points += theCharacter.shield.value;
        }
        return [ name, armour, theCharacter.woundPoints[location].toString(), points.toString() ];
    };
    content.push(locline("Head", "head"));
    content.push(locline("Shield Arm", "shieldArm"));
    content.push(locline("Sword Arm", "swordArm"));
    content.push(locline("Chest", "chest"));
    content.push(locline("Abdomen", "abdomen"));
    content.push(locline("Right Leg", "rightLeg"));
    content.push(locline("Left Leg", "leftLeg"));
    Character.PDFdrawTable(pdf, content, [ 64, 100, 32, 32 ], [ "left", "left", "center", "center" ], 405, 410, layout.cellheight, [ 1, 1, 1, 0.5, 1, 0.5 ]);

    pdf.setLineWidth(1);
    pdf.rect(633, 420, 6 * layout.cellwidth, 7 * layout.cellheight);
    pdf.setLineWidth(0.5);
    for (i = 1; i < 7; i++) {
        pdf.line(633, 420 + i * layout.cellheight, 633 + 6 * layout.cellwidth, 420 + i * layout.cellheight);
    }
    for (i = 1; i < 6; i++) {
        pdf.line(633 + i * layout.cellwidth, 420, 633 + i * layout.cellwidth, 420 + 7 * layout.cellheight);
    }

    content = [];
    content.push([ "Type of encumbrance", "Modifier" ]);
    content.push([ "Light encumbrance", "" ]);
    content.push([ "Medium encumbrance", "-1" ]);
    content.push([ "Heavy encumbrance", "-3" ]);
    Character.PDFdrawTable(pdf, content, modwidths, [ "left", "center" ], 405, 497, layout.cellheight);

    content = [];
    content.push([ "Movement", "Speed", "Modifier" ]);
    content.push([ "Walk", ""+Math.floor(5+theCharacter.agi/2), "0" ]);
    content.push([ "Jog", ""+Math.floor(10+theCharacter.agi), "-2" ]);
    content.push([ "Sprint", ""+Math.floor(10+3*theCharacter.agi), "-4" ]);
    Character.PDFdrawTable(pdf, content, [ 64, 32, 64 ], [ "left", "center", "center" ], 570, 497, layout.cellheight);

    if (hasarea) {
        pdf.text(738, 502, "Area weapons can hit");
        pdf.text(738, 510, "additional targets in the");
        pdf.text(738, 518, "area with a -2 modifier. Each");
        pdf.text(738, 526, "target can be hit multiple");
        pdf.text(738, 534, "times up to the hits number");
        pdf.text(738, 542, "with one extra hit for each");
        pdf.text(738, 550, "level of victory above minor.");
    } else if (hashits) {
        pdf.text(738, 502, "Each target can be hit multiple");
        pdf.text(738, 510, "times up to the hits number");
        pdf.text(738, 518, "with one extra hit for every");
        pdf.text(738, 525, "level of victory above minor.");
    }

    Character.PDFfooter(pdf,layout,theCharacter);
};

Character.makeSkillTablesPage = function(pdf,layout,theCharacter) {
    pdf.setFontSize(18);
    pdf.textInBox(theCharacter.name, {
        align : "center"
    }, layout.nameleft, layout.namebottom, layout.nameright, layout.nametop);

    var untrained = [];
    untrained.push({
        "skill" : "All untrained skills",
        "level" : 0
    });
    Character.PDFdrawSkillTable(pdf, "Skill", untrained, layout.namebottom+8, layout.left, layout.cellheight, layout.cellwidth);
    Character.PDFdrawSkillTable(pdf, "Skill", theCharacter.sheet.skills, layout.namebottom+54, layout.left, layout.cellheight, layout.cellwidth);

    Character.PDFfooter(pdf,layout,theCharacter);
};